package com.ghomerr.linkedchest.listeners;

import java.util.List;
import java.util.logging.Logger;

import org.bukkit.ChatColor;
import org.bukkit.block.Block;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.block.BlockBurnEvent;
import org.bukkit.event.block.BlockPlaceEvent;
import org.bukkit.event.block.SignChangeEvent;
import org.bukkit.event.entity.EntityExplodeEvent;

import com.ghomerr.linkedchest.LinkedChest;
import com.ghomerr.linkedchest.constants.Constants;
import com.ghomerr.linkedchest.enums.Messages;
import com.ghomerr.linkedchest.enums.PermissionsNodes;
import com.ghomerr.linkedchest.utils.MessagesUtils;
import com.ghomerr.linkedchest.utils.PermissionsUtils;
import com.ghomerr.linkedchest.utils.StringUtils;
import com.ghomerr.linkedchest.utils.WorldUtils;

public class BlockEventListener implements Listener
{
	private static final Logger _LOGGER = Logger.getLogger(Constants.MINECRAFT);

	public LinkedChest plugin = null;

	public BlockEventListener (final LinkedChest plugin)
	{
		this.plugin = plugin;
		this.plugin.getServer().getPluginManager().registerEvents(this, plugin);
	}

	@EventHandler
	public void onBlockBreakEvent(final BlockBreakEvent event)
	{
		final String chestName = preventMasterChestToBeBroken(event.getBlock());
		if (chestName != null)
		{
			event.setCancelled(true);
			event.getPlayer().sendMessage(
					MessagesUtils.getWithColors(Messages.MASTER_CHEST_UNBREAKABLE, ChatColor.RED, ChatColor.AQUA,
							chestName));
			_LOGGER.info(Constants.TAG + "Chest '" + chestName + "' has been saved from being broken by "
					+ event.getPlayer().getName());
		}
	}

	@EventHandler
	public void onEntityExplodeEvent(final EntityExplodeEvent event)
	{
		final List<Block> blockList = event.blockList();
		for (final Block block : blockList)
		{
			final String chestName = preventMasterChestToBeBroken(block);
			if (chestName != null)
			{
				event.setCancelled(true);
				_LOGGER.info(Constants.TAG + "Chest '" + chestName + "' has been saved from explosion caused by "
						+ event.getEntity() + " at " + StringUtils.printShortLocation(event.getLocation()));
				break;
			}
		}
	}

	@EventHandler
	public void onBlockBurnEvent(final BlockBurnEvent event)
	{
		final String chestName = preventMasterChestToBeBroken(event.getBlock());
		if (chestName != null)
		{
			event.setCancelled(true);
			_LOGGER.info(Constants.TAG + "Chest '" + chestName + "' has been saved from burning");
		}
	}

	@EventHandler
	public void onBlockPlaceEvent(final BlockPlaceEvent event)
	{
		final Block placedBlock = event.getBlockPlaced();
		if (WorldUtils.isChest(placedBlock))
		{
			final Block block = WorldUtils.getChestNearby(placedBlock, Block.class);
			if (block != null)
			{
				final String chestName = plugin.getMasterChestNameFromBlock(block);
				if (chestName != null)
				{
					event.getPlayer().sendMessage(
							MessagesUtils.getWithColors(Messages.CANNOT_PLACE_CHEST_NEARBY, ChatColor.RED, ChatColor.AQUA,
									chestName));
					event.setCancelled(true);
				}
			}
		}
	}

	@EventHandler
	public void onSignChange(final SignChangeEvent event)
	{
		final Player player = event.getPlayer();

		final String chestName = WorldUtils.getChestNameOnSign(event.getLines());

		if (chestName != null)
		{
			if (plugin.hasChestName(chestName))
			{
				if (!PermissionsUtils.hasPermission(player, PermissionsNodes.PLACE_LINKED_SIGN))
				{
					event.setCancelled(true);
				}
			}
		}
	}

	private String preventMasterChestToBeBroken(final Block block)
	{
		String chestName = null;
		if (WorldUtils.isChest(block))
		{
			chestName = plugin.getMasterChestNameFromBlock(block);
			if (StringUtils.isBlank(chestName))
			{
				chestName = null;
			}
		}
		return chestName;
	}
}
